

{- |
Copyright  : Will Thompson, Iñaki García Etxebarria and Jonas Platte
License    : LGPL-2.1
Maintainer : Iñaki García Etxebarria (garetxe@gmail.com)

Information about a method on an D-Bus interface.
-}

module GI.Gio.Structs.DBusMethodInfo
    ( 

-- * Exported types
    DBusMethodInfo(..)                      ,
    newZeroDBusMethodInfo                   ,
    noDBusMethodInfo                        ,


 -- * Methods
-- ** dBusMethodInfoRef
    DBusMethodInfoRefMethodInfo             ,
    dBusMethodInfoRef                       ,


-- ** dBusMethodInfoUnref
    DBusMethodInfoUnrefMethodInfo           ,
    dBusMethodInfoUnref                     ,




 -- * Properties
-- ** Annotations
    dBusMethodInfoReadAnnotations           ,


-- ** InArgs
    dBusMethodInfoReadInArgs                ,


-- ** Name
    dBusMethodInfoReadName                  ,


-- ** OutArgs
    dBusMethodInfoReadOutArgs               ,


-- ** RefCount
    dBusMethodInfoReadRefCount              ,




    ) where

import Prelude ()
import Data.GI.Base.ShortPrelude

import qualified Data.Text as T
import qualified Data.ByteString.Char8 as B
import qualified Data.Map as Map

import GI.Gio.Types
import GI.Gio.Callbacks

newtype DBusMethodInfo = DBusMethodInfo (ForeignPtr DBusMethodInfo)
foreign import ccall "g_dbus_method_info_get_type" c_g_dbus_method_info_get_type :: 
    IO GType

instance BoxedObject DBusMethodInfo where
    boxedType _ = c_g_dbus_method_info_get_type

-- | Construct a `DBusMethodInfo` struct initialized to zero.
newZeroDBusMethodInfo :: MonadIO m => m DBusMethodInfo
newZeroDBusMethodInfo = liftIO $ callocBoxedBytes 40 >>= wrapBoxed DBusMethodInfo

noDBusMethodInfo :: Maybe DBusMethodInfo
noDBusMethodInfo = Nothing

dBusMethodInfoReadRefCount :: DBusMethodInfo -> IO Int32
dBusMethodInfoReadRefCount s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 0) :: IO Int32
    return val

dBusMethodInfoReadName :: DBusMethodInfo -> IO T.Text
dBusMethodInfoReadName s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 8) :: IO CString
    val' <- cstringToText val
    return val'

dBusMethodInfoReadInArgs :: DBusMethodInfo -> IO [DBusArgInfo]
dBusMethodInfoReadInArgs s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 16) :: IO (Ptr (Ptr DBusArgInfo))
    val' <- unpackZeroTerminatedPtrArray val
    val'' <- mapM (newBoxed DBusArgInfo) val'
    return val''

dBusMethodInfoReadOutArgs :: DBusMethodInfo -> IO [DBusArgInfo]
dBusMethodInfoReadOutArgs s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 24) :: IO (Ptr (Ptr DBusArgInfo))
    val' <- unpackZeroTerminatedPtrArray val
    val'' <- mapM (newBoxed DBusArgInfo) val'
    return val''

dBusMethodInfoReadAnnotations :: DBusMethodInfo -> IO [DBusAnnotationInfo]
dBusMethodInfoReadAnnotations s = withManagedPtr s $ \ptr -> do
    val <- peek (ptr `plusPtr` 32) :: IO (Ptr (Ptr DBusAnnotationInfo))
    val' <- unpackZeroTerminatedPtrArray val
    val'' <- mapM (newBoxed DBusAnnotationInfo) val'
    return val''

-- method DBusMethodInfo::ref
-- method type : OrdinaryMethod
-- Args : [Arg {argCName = "_obj", argType = TInterface "Gio" "DBusMethodInfo", direction = DirectionIn, mayBeNull = False, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, argCallerAllocates = False, transfer = TransferNothing}]
-- Lengths : []
-- returnType : TInterface "Gio" "DBusMethodInfo"
-- throws : False
-- Skip return : False

foreign import ccall "g_dbus_method_info_ref" g_dbus_method_info_ref :: 
    Ptr DBusMethodInfo ->                   -- _obj : TInterface "Gio" "DBusMethodInfo"
    IO (Ptr DBusMethodInfo)


dBusMethodInfoRef ::
    (MonadIO m) =>
    DBusMethodInfo                          -- _obj
    -> m DBusMethodInfo                     -- result
dBusMethodInfoRef _obj = liftIO $ do
    let _obj' = unsafeManagedPtrGetPtr _obj
    result <- g_dbus_method_info_ref _obj'
    checkUnexpectedReturnNULL "g_dbus_method_info_ref" result
    result' <- (wrapBoxed DBusMethodInfo) result
    touchManagedPtr _obj
    return result'

data DBusMethodInfoRefMethodInfo
instance (signature ~ (m DBusMethodInfo), MonadIO m) => MethodInfo DBusMethodInfoRefMethodInfo DBusMethodInfo signature where
    overloadedMethod _ = dBusMethodInfoRef

-- method DBusMethodInfo::unref
-- method type : OrdinaryMethod
-- Args : [Arg {argCName = "_obj", argType = TInterface "Gio" "DBusMethodInfo", direction = DirectionIn, mayBeNull = False, argScope = ScopeTypeInvalid, argClosure = -1, argDestroy = -1, argCallerAllocates = False, transfer = TransferNothing}]
-- Lengths : []
-- returnType : TBasicType TVoid
-- throws : False
-- Skip return : False

foreign import ccall "g_dbus_method_info_unref" g_dbus_method_info_unref :: 
    Ptr DBusMethodInfo ->                   -- _obj : TInterface "Gio" "DBusMethodInfo"
    IO ()


dBusMethodInfoUnref ::
    (MonadIO m) =>
    DBusMethodInfo                          -- _obj
    -> m ()                                 -- result
dBusMethodInfoUnref _obj = liftIO $ do
    let _obj' = unsafeManagedPtrGetPtr _obj
    g_dbus_method_info_unref _obj'
    touchManagedPtr _obj
    return ()

data DBusMethodInfoUnrefMethodInfo
instance (signature ~ (m ()), MonadIO m) => MethodInfo DBusMethodInfoUnrefMethodInfo DBusMethodInfo signature where
    overloadedMethod _ = dBusMethodInfoUnref

type family ResolveDBusMethodInfoMethod (t :: Symbol) (o :: *) :: * where
    ResolveDBusMethodInfoMethod "ref" o = DBusMethodInfoRefMethodInfo
    ResolveDBusMethodInfoMethod "unref" o = DBusMethodInfoUnrefMethodInfo
    ResolveDBusMethodInfoMethod l o = MethodResolutionFailed l o

instance (info ~ ResolveDBusMethodInfoMethod t DBusMethodInfo, MethodInfo info DBusMethodInfo p) => IsLabelProxy t (DBusMethodInfo -> p) where
    fromLabelProxy _ = overloadedMethod (MethodProxy :: MethodProxy info)

#if MIN_VERSION_base(4,9,0)
instance (info ~ ResolveDBusMethodInfoMethod t DBusMethodInfo, MethodInfo info DBusMethodInfo p) => IsLabel t (DBusMethodInfo -> p) where
    fromLabel _ = overloadedMethod (MethodProxy :: MethodProxy info)
#endif


